@@ -53,11 +53,12 @@ from ansible.module_utils.ansible_release import __version__ as ansible_version
 from ansible.module_utils.common.collections import is_iterable
 from ansible.module_utils.common.yaml import yaml_dump, yaml_load
 from ansible.module_utils.common.text.converters import to_bytes, to_native, to_text
+from ansible._internal._datatag._tags import TrustedAsTemplate
 from ansible.module_utils import six
 from ansible.parsing.dataloader import DataLoader
-from ansible.parsing.yaml.loader import AnsibleLoader
 from ansible.playbook.role.requirement import RoleRequirement
-from ansible.template import Templar
+from ansible._internal._templating._engine import TemplateEngine
+from ansible.template import trust_as_template
 from ansible.utils.collection_loader import AnsibleCollectionConfig
 from ansible.utils.display import Display
 from ansible.utils.plugin_docs import get_versioned_doclink
@@ -66,27 +67,6 @@ from ansible.utils.vars import load_extra_vars
 display = Display()
 urlparse = six.moves.urllib.parse.urlparse
 
-# config definition by position: name, required, type
-SERVER_DEF = [
-    ('url', True, 'str'),
-    ('username', False, 'str'),
-    ('password', False, 'str'),
-    ('token', False, 'str'),
-    ('auth_url', False, 'str'),
-    ('api_version', False, 'int'),
-    ('validate_certs', False, 'bool'),
-    ('client_id', False, 'str'),
-    ('timeout', False, 'int'),
-]
-
-# config definition fields
-SERVER_ADDITIONAL = {
-    'api_version': {'default': None, 'choices': [2, 3]},
-    'validate_certs': {'cli': [{'name': 'validate_certs'}]},
-    'timeout': {'default': C.GALAXY_SERVER_TIMEOUT, 'cli': [{'name': 'timeout'}]},
-    'token': {'default': None},
-}
-
 
 def with_collection_artifacts_manager(wrapped_method):
     """Inject an artifacts manager if not passed explicitly.
@@ -199,11 +179,11 @@ class RoleDistributionServer:
 
 
 class GalaxyCLI(CLI):
-    '''Command to manage Ansible roles and collections.
+    """Command to manage Ansible roles and collections.
 
        None of the CLI tools are designed to run concurrently with themselves.
        Use an external scheduler and/or locking to ensure there are no clashing operations.
-    '''
+    """
 
     name = 'ansible-galaxy'
 
@@ -234,7 +214,7 @@ class GalaxyCLI(CLI):
         super(GalaxyCLI, self).__init__(args)
 
     def init_parser(self):
-        ''' create an options parser for bin/ansible '''
+        """ create an options parser for bin/ansible """
 
         super(GalaxyCLI, self).init_parser(
             desc="Perform various Role and Collection related operations.",
@@ -641,25 +621,8 @@ class GalaxyCLI(CLI):
 
         self.galaxy = Galaxy()
 
-        def server_config_def(section, key, required, option_type):
-            config_def = {
-                'description': 'The %s of the %s Galaxy server' % (key, section),
-                'ini': [
-                    {
-                        'section': 'galaxy_server.%s' % section,
-                        'key': key,
-                    }
-                ],
-                'env': [
-                    {'name': 'ANSIBLE_GALAXY_SERVER_%s_%s' % (section.upper(), key.upper())},
-                ],
-                'required': required,
-                'type': option_type,
-            }
-            if key in SERVER_ADDITIONAL:
-                config_def.update(SERVER_ADDITIONAL[key])
-
-            return config_def
+        # dynamically add per server config depending on declared servers
+        C.config.load_galaxy_server_defs(C.GALAXY_SERVER_LIST)
 
         galaxy_options = {}
         for optional_key in ['clear_response_cache', 'no_cache']:
@@ -667,24 +630,18 @@ class GalaxyCLI(CLI):
                 galaxy_options[optional_key] = context.CLIARGS[optional_key]
 
         config_servers = []
-
         # Need to filter out empty strings or non truthy values as an empty server list env var is equal to [''].
         server_list = [s for s in C.GALAXY_SERVER_LIST or [] if s]
         for server_priority, server_key in enumerate(server_list, start=1):
-            # Abuse the 'plugin config' by making 'galaxy_server' a type of plugin
-            # Config definitions are looked up dynamically based on the C.GALAXY_SERVER_LIST entry. We look up the
-            # section [galaxy_server.<server>] for the values url, username, password, and token.
-            config_dict = dict((k, server_config_def(server_key, k, req, ensure_type)) for k, req, ensure_type in SERVER_DEF)
-            defs = AnsibleLoader(yaml_dump(config_dict)).get_single_data()
-            C.config.initialize_plugin_configuration_definitions('galaxy_server', server_key, defs)
 
             # resolve the config created options above with existing config and user options
-            server_options = C.config.get_plugin_options('galaxy_server', server_key)
+            server_options = C.config.get_plugin_options(plugin_type='galaxy_server', name=server_key)
 
             # auth_url is used to create the token, but not directly by GalaxyAPI, so
             # it doesn't need to be passed as kwarg to GalaxyApi, same for others we pop here
             auth_url = server_options.pop('auth_url')
             client_id = server_options.pop('client_id')
+            client_secret = server_options.pop('client_secret')
             token_val = server_options['token'] or NoTokenSentinel
             username = server_options['username']
             api_version = server_options.pop('api_version')
@@ -710,15 +667,17 @@ class GalaxyCLI(CLI):
             if username:
                 server_options['token'] = BasicAuthToken(username, server_options['password'])
             else:
-                if token_val:
-                    if auth_url:
-                        server_options['token'] = KeycloakToken(access_token=token_val,
-                                                                auth_url=auth_url,
-                                                                validate_certs=validate_certs,
-                                                                client_id=client_id)
-                    else:
-                        # The galaxy v1 / github / django / 'Token'
-                        server_options['token'] = GalaxyToken(token=token_val)
+                if auth_url:
+                    server_options['token'] = KeycloakToken(
+                        access_token=token_val,
+                        auth_url=auth_url,
+                        validate_certs=validate_certs,
+                        client_id=client_id,
+                        client_secret=client_secret,
+                    )
+                elif token_val:
+                    # The galaxy v1 / github / django / 'Token'
+                    server_options['token'] = GalaxyToken(token=token_val)
 
             server_options.update(galaxy_options)
             config_servers.append(GalaxyAPI(
@@ -958,8 +917,8 @@ class GalaxyCLI(CLI):
 
     @staticmethod
     def _get_skeleton_galaxy_yml(template_path, inject_data):
-        with open(to_bytes(template_path, errors='surrogate_or_strict'), 'rb') as template_obj:
-            meta_template = to_text(template_obj.read(), errors='surrogate_or_strict')
+        with open(to_bytes(template_path, errors='surrogate_or_strict'), 'r') as template_obj:
+            meta_template = TrustedAsTemplate().tag(to_text(template_obj.read(), errors='surrogate_or_strict'))
 
         galaxy_meta = get_collections_galaxy_meta_info()
 
@@ -995,7 +954,7 @@ class GalaxyCLI(CLI):
             return textwrap.fill(v, width=117, initial_indent="# ", subsequent_indent="# ", break_on_hyphens=False)
 
         loader = DataLoader()
-        templar = Templar(loader, variables={'required_config': required_config, 'optional_config': optional_config})
+        templar = TemplateEngine(loader, variables={'required_config': required_config, 'optional_config': optional_config})
         templar.environment.filters['comment_ify'] = comment_ify
 
         meta_value = templar.template(meta_template)
@@ -1197,7 +1156,7 @@ class GalaxyCLI(CLI):
 
         loader = DataLoader()
         inject_data.update(load_extra_vars(loader))
-        templar = Templar(loader, variables=inject_data)
+        templar = TemplateEngine(loader, variables=inject_data)
 
         # create role directory
         if not os.path.exists(b_obj_path):
@@ -1239,7 +1198,7 @@ class GalaxyCLI(CLI):
                 elif ext == ".j2" and not in_templates_dir:
                     src_template = os.path.join(root, f)
                     dest_file = os.path.join(obj_path, rel_root, filename)
-                    template_data = to_text(loader._get_file_contents(src_template)[0], errors='surrogate_or_strict')
+                    template_data = trust_as_template(loader.get_text_file_contents(src_template))
                     try:
                         b_rendered = to_bytes(templar.template(template_data), errors='surrogate_or_strict')
                     except AnsibleError as e:
@@ -1767,7 +1726,7 @@ class GalaxyCLI(CLI):
         publish_collection(collection_path, self.api, wait, timeout)
 
     def execute_search(self):
-        ''' searches for roles on the Ansible Galaxy server'''
+        """ searches for roles on the Ansible Galaxy server"""
         page_size = 1000
         search = None
 
@@ -1807,6 +1766,8 @@ class GalaxyCLI(CLI):
 
         return 0
 
+    _task_check_delay_sec = 10  # allows unit test override
+
     def execute_import(self):
         """ used to import a role into Ansible Galaxy """
 
@@ -1860,7 +1821,7 @@ class GalaxyCLI(CLI):
                     rc = ['SUCCESS', 'FAILED'].index(state)
                     finished = True
                 else:
-                    time.sleep(10)
+                    time.sleep(self._task_check_delay_sec)
 
         return rc
 
