@@ -3,16 +3,16 @@
 
 from __future__ import annotations
 
-DOCUMENTATION = r'''
+DOCUMENTATION = r"""
     name: host_list
     version_added: "2.4"
     short_description: Parses a 'host list' string
     description:
         - Parses a host list string as a comma separated values of hosts
         - This plugin only applies to inventory strings that are not paths and contain a comma.
-'''
+"""
 
-EXAMPLES = r'''
+EXAMPLES = r"""
     # define 2 hosts in command line
     # ansible -i '10.10.2.6, 10.10.2.4' -m ping all
 
@@ -21,7 +21,7 @@ EXAMPLES = r'''
 
     # just use localhost
     # ansible-playbook -i 'localhost,' play.yml -c local
-'''
+"""
 
 import os
 
@@ -35,6 +35,8 @@ class InventoryModule(BaseInventoryPlugin):
 
     NAME = 'host_list'
 
+    # host_list does not set vars, so needs no special trust assistance from the inventory API
+
     def verify_file(self, host_list):
 
         valid = False
@@ -44,7 +46,7 @@ class InventoryModule(BaseInventoryPlugin):
         return valid
 
     def parse(self, inventory, loader, host_list, cache=True):
-        ''' parses the inventory file '''
+        """ parses the inventory file """
 
         super(InventoryModule, self).parse(inventory, loader, host_list)
 
