@@ -2,12 +2,11 @@
 
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 
 class ModuleDocFragment(object):
-    DOCUMENTATION = r'''
+    DOCUMENTATION = r"""
 options:
     testcol2option:
         description:
@@ -18,4 +17,4 @@ options:
         description:
             - Another option taken from testcol2
         type: str
-'''
+"""
