@@ -1,21 +1,11 @@
 from __future__ import annotations
 
-import json
-import sys
 import time
 
 from ansible.module_utils.basic import AnsibleModule
 
 
 def main():
-    if "--interactive" in sys.argv:
-        import ansible.module_utils.basic
-        ansible.module_utils.basic._ANSIBLE_ARGS = json.dumps(dict(
-            ANSIBLE_MODULE_ARGS=dict(
-                fail_mode="graceful"
-            )
-        ))
-
     module = AnsibleModule(
         argument_spec=dict(
             fail_mode=dict(type='list', default=['success'])
