@@ -2,16 +2,16 @@ from __future__ import annotations
 
 
 class ModuleDocFragment(object):
-    DOCUMENTATION = r'''
+    DOCUMENTATION = r"""
 options:
   normal_doc_frag:
     description:
     - an option
-'''
+"""
 
-    OTHER_DOCUMENTATION = r'''
+    OTHER_DOCUMENTATION = r"""
 options:
   other_doc_frag:
     description:
     - another option
-'''
+"""
