@@ -4,8 +4,8 @@ import re
 
 
 def test_name_ok(value):
-    # test names are prefixed with a unique hash value to prevent shadowing of other plugins
-    return bool(re.match(r'^ansible\.plugins\.test\.[0-9]+_test_test$', __name__))
+    # test names include a unique hash value to prevent shadowing of other plugins
+    return bool(re.match(r'^ansible\.plugins\.test\.test_test_[0-9]+$', __name__))
 
 
 class TestModule:
