@@ -3,10 +3,7 @@
 set -eux
 
 # this should succeed since we override the undefined variable
-ansible-playbook undefined.yml -i inventory -v "$@" -e '{"mytest": False}'
-
-# this should still work, just show that var is undefined in debug
-ansible-playbook undefined.yml -i inventory -v "$@"
+ansible-playbook undefined.yml -i inventory -v "$@" -e '{"override_value": "overridden by -e"}'
 
 # this should work since we dont use the variable
 ansible-playbook undall.yml -i inventory -v "$@"
@@ -16,6 +13,3 @@ ansible-playbook task_vars_templating.yml -v "$@"
 
 # there should be an attempt to use 'sudo' in the connection debug output
 ANSIBLE_BECOME_ALLOW_SAME_USER=true ansible-playbook test_connection_vars.yml -vvvv "$@" | tee /dev/stderr | grep 'sudo \-H \-S'
-
-# smoke test usage of VarsWithSources that is used when ANSIBLE_DEBUG=1
-ANSIBLE_DEBUG=1 ansible-playbook test_vars_with_sources.yml -v "$@"
