@@ -6,15 +6,12 @@
 
 from __future__ import annotations
 
-from units.mock.procenv import ModuleTestCase
+import unittest
 
 from unittest.mock import patch
-import builtins
-
-realimport = builtins.__import__
 
 
-class TestGetModulePath(ModuleTestCase):
+class TestGetModulePath(unittest.TestCase):
     def test_module_utils_basic_get_module_path(self):
         from ansible.module_utils.basic import get_module_path
         with patch('os.path.realpath', return_value='/path/to/foo/'):
