@@ -1,10 +1,9 @@
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import os
 
-from units.compat.mock import patch, Mock
-from units.compat import unittest
+from unittest.mock import patch, Mock
+import unittest
 
 from ansible.modules import apt_key
 
